
//------------------------------------------------------------------------------
// File: VirtualSource.cpp
//
// Desc: DirectShow sample code - implementation of filter behaviors
//       for the virtual source filter.
//		
//		 The filter includes both a CSource object and a CSourceStream for the output pin
//		 Observe that only the render method has to be changed to create another stream
//
//------------------------------------------------------------------------------
#include <windows.h>
#include <streams.h>
#include <olectl.h>
#include <initguid.h>
#include "VirtualSource.h"

//#include "VirtualSourceProp.h"


#pragma warning(disable:4710)  // 'function': function not inlined (optimzation)

// Forward declarations
LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );


// Setup data

WNDCLASSEX wc;

// A structure for our custom vertex type. We added a normal, and omitted the
// color (which is provided by the material)
struct CUSTOMVERTEX
{
    FLOAT x,y,z;		// The 3D position for the vertex
    FLOAT nx,ny,nz;		// The surface normal for the vertex
    FLOAT tu, tv;		// The texture coordinates
};

// Our custom FVF, which describes our custom vertex structure
#define D3DFVF_CUSTOMVERTEX (D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_TEX1)

const AMOVIESETUP_MEDIATYPE sudOpPinTypes =
{
    &MEDIATYPE_Video,       // Major type
    &MEDIASUBTYPE_NULL      // Minor type
};

const AMOVIESETUP_PIN sudOpPin =
{
    L"Output",              // Pin string name
    FALSE,                  // Is it rendered
    TRUE,                   // Is it an output
    FALSE,                  // Can we have none
    FALSE,                  // Can we have many
    &CLSID_NULL,            // Connects to filter
    NULL,                   // Connects to pin
    1,                      // Number of types
    &sudOpPinTypes };       // Pin details

const AMOVIESETUP_FILTER sudVirtualSourceax =
{
    &CLSID_CVirtualSource,    // Filter CLSID
    L"VirtualSource",       // String name
    MERIT_DO_NOT_USE,       // Filter merit
    1,                      // Number pins
    &sudOpPin               // Pin details
};


// COM global table of objects in this dll

CFactoryTemplate g_Templates[1] = {
  { L"VirtualSource"
  , &CLSID_CVirtualSource
  , CVirtualSource::CreateInstance
  , NULL
  , &sudVirtualSourceax }
 /* ,
  { L"VirtualSource Property Page"
  , &CLSID_VirtualSourcePropertyPage
  , CVirtualSourceProperties::CreateInstance }*/
};
int g_cTemplates = sizeof(g_Templates) / sizeof(g_Templates[0]);

//
// DllRegisterServer
//
// Exported entry points for registration and unregistration
//
STDAPI DllRegisterServer()
{
    return AMovieDllRegisterServer2( TRUE );

} // DllRegisterServer


//
// DllUnregisterServer
//
STDAPI DllUnregisterServer()
{
    return AMovieDllRegisterServer2( FALSE );

} // DllUnregisterServer


//
// CreateInstance
//
// The only allowed way to create the Virtual Source
//
CUnknown * WINAPI CVirtualSource::CreateInstance(LPUNKNOWN lpunk, HRESULT *phr)
{
    CUnknown *punk = new CVirtualSource(lpunk, phr);
    if (punk == NULL) {
        *phr = E_OUTOFMEMORY;
    }
    return punk;

} // CreateInstance


//
// Constructor
//
// Initialise a CVirtualStream object so that we have a pin.
//
CVirtualSource::CVirtualSource(LPUNKNOWN lpunk, HRESULT *phr) :
    CSource(NAME("VirtualSource"),
            lpunk,
            CLSID_CVirtualSource)
{
    CAutoLock cAutoLock(&m_cStateLock);

    m_paStreams    = (CSourceStream **) new CVirtualStream*[1];
    if (m_paStreams == NULL) {
        *phr = E_OUTOFMEMORY;
	return;
    }

    m_paStreams[0] = new CVirtualStream(phr, this, L"Virtual output");
    if (m_paStreams[0] == NULL) {
        *phr = E_OUTOFMEMORY;
	return;
    }

} // (Constructor)

	/*
//
// NonDelegatingQueryInterface
//
// Reveals IContrast and ISpecifyPropertyPages
//
STDMETHODIMP CVirtualSource::NonDelegatingQueryInterface(REFIID riid, void **ppv)
{
    CheckPointer(ppv,E_POINTER);

    if (riid == IID_ISpecifyPropertyPages) {
        return GetInterface((ISpecifyPropertyPages *) this, ppv);
    } else {
        return CSource::NonDelegatingQueryInterface(riid, ppv);
    }

} // NonDelegatingQueryInterface

//
// GetPages
//
// This is the sole member of ISpecifyPropertyPages
// Returns the clsid's of the property pages we support
//
STDMETHODIMP CVirtualSource::GetPages(CAUUID *pPages)
{
    pPages->cElems = 1;
    pPages->pElems = (GUID *) CoTaskMemAlloc(sizeof(GUID));
    if (pPages->pElems == NULL) {
        return E_OUTOFMEMORY;
    }
    *(pPages->pElems) = CLSID_VirtualSourcePropertyPage;
    return NOERROR;

} // GetPages
*/

//
// Constructor
//
CVirtualStream::CVirtualStream(HRESULT *phr,
                         CVirtualSource *pParent,
                         LPCWSTR pPinName) :
    CSourceStream(NAME("VirtualSource"),phr, pParent, pPinName),
    m_iImageWidth(VID_WIDTH),
    m_iImageHeight(VID_HEIGHT),
    m_iDefaultRepeatTime(20)
{
		
    CAutoLock cAutoLock(&m_cSharedState);

	// create logfile
	m_logfile = new ofstream(LOGFILE_LOCATION);

		//CREATE WINDOW
    // Register the window class
    WNDCLASSEX i_wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L,
                      GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
                      "D3D Filter", NULL };
	m_wc = wc = i_wc;

	if(FAILED(RegisterClassEx( &m_wc )))
		*m_logfile << LOCATION << "register failed";
 
	// Create the application's window
	hWnd = CreateWindow( "D3D Filter", "D3D Filter: Render window",
                              WS_OVERLAPPED, 100, 100, 300, 300,
                              GetDesktopWindow(), NULL, NULL, NULL );
	 
	if(hWnd == NULL)
		*m_logfile << LOCATION << "hWnd is NULL. Error is: " << GetLastError() << ".\n";

	// INITIALIZE D3D device
	if( FAILED( this->InitD3D(hWnd) ) )
		*m_logfile << LOCATION << "initializing of d3d failed";	

	if(FAILED(D3DXCreateRenderToSurface(m_pd3dDevice, m_iImageWidth, m_iImageHeight, 
        D3DFMT_X8R8G8B8, FALSE, D3DFMT_UNKNOWN, &m_pRenderToSurface)))
        *m_logfile << LOCATION << "initializing of Renderer to Surface failed";

	if(FAILED(m_pd3dDevice->CreateImageSurface(m_iImageWidth, m_iImageHeight, 
        D3DFMT_X8R8G8B8, &m_pRenderTargetSurfaceLeft)))
        *m_logfile << LOCATION << "initializing of Render Target Surface failed";

	if(FAILED(m_pd3dDevice->CreateImageSurface(m_iImageWidth, m_iImageHeight, 
        D3DFMT_X8R8G8B8, &m_pRenderTargetSurfaceRight)))
        *m_logfile << LOCATION << "initializing of Render Target Surface failed";
 
	if(FAILED(InitGeometry()))
        *m_logfile << LOCATION << "Could not initialize the geometry";

	m_logfile->flush();

} // (Constructor)


//
// Destructor
//
CVirtualStream::~CVirtualStream()
{
    CAutoLock cAutoLock(&m_cSharedState);


} // (Destructor)

//-----------------------------------------------------------------------------
// Name: InitD3D()
// Desc: Initializes Direct3D
//-----------------------------------------------------------------------------
HRESULT CVirtualStream::InitD3D( HWND hWnd )
{
    // Create the D3D object.
    if( NULL == ( m_pD3D = Direct3DCreate8( D3D_SDK_VERSION ) ) )
        return E_FAIL;

    // Get the current desktop display mode, so we can set up a back
    // buffer of the same format
    D3DDISPLAYMODE d3ddm;
    if( FAILED( m_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm ) ) )
	{
        *m_logfile << LOCATION << "Could not get adapter display mode";
        return E_FAIL;
	}

    // Set up the structure used to create the D3DDevice. Since we are now
    // using more complex geometry, we will create a device with a zbuffer.
    D3DPRESENT_PARAMETERS d3dpp;
    ZeroMemory( &d3dpp, sizeof(d3dpp) );
    d3dpp.Windowed = TRUE;
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat = d3ddm.Format;
    d3dpp.EnableAutoDepthStencil = TRUE;
    d3dpp.AutoDepthStencilFormat = D3DFMT_D16;

    // Create the D3DDevice
    if( FAILED( m_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_HARDWARE_VERTEXPROCESSING|D3DCREATE_MULTITHREADED,
                                      &d3dpp, &m_pd3dDevice ) ) )
    {
		
        *m_logfile << LOCATION << "Could not create device";
        return E_FAIL;
    }

    // Turn off culling
    m_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );

    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: InitGeometry()
// Desc: Creates the scene geometry
//-----------------------------------------------------------------------------
HRESULT CVirtualStream::InitGeometry()
{

    HRESULT hr;

	// Use D3DX to create a texture from a file based image
    if( FAILED(  hr =  D3DXCreateTextureFromFile(m_pd3dDevice, TEXTURE_LOCATION,
                                           &m_pTexture ) ) )
    {
        *m_logfile << LOCATION << "Could not create texture from file";
        return hr;
    }
	// Use D3DX to create a texture from a file based image
    if( FAILED(  hr =  D3DXCreateTextureFromFile(m_pd3dDevice, ENVIRONMENT_LOCATION,
                                           &m_pTexture_environment ) ) )
    {
        *m_logfile << LOCATION << "Could not create texture from file";
        return hr;
    }

    // Initialize three vertices for rendering a star
    CUSTOMVERTEX m_Vertices[] =

	{	/*{-0.8f,-1.0f, 1.0f, 0.0f, 0.0f,1.0f, 0.1f, 0.0f }, //1
		{ 0.0f,-0.4f, 1.0f, 0.0f, 0.0f,1.0f, 0.5f, 0.3f }, //2
		{ 0.0f, 0.0f, 0.7f, 0.0f, 0.0f,1.0f, 0.5f, 0.5f },
		{-0.8f,-1.0f, 1.0f, 0.0f, 0.0f,1.0f, 0.1f, 0.0f }, //1
		{-0.4f,-0.2f, 1.0f, 0.0f, 0.0f,1.0f, 0.3f, 0.4f }, //4
		{ 0.0f, 0.0f, 0.7f, 0.0f, 0.0f,1.0f, 0.5f, 0.5f },
        { 0.8f,-1.0f, 1.0f, 0.0f, 0.0f,1.0f, 0.9f, 0.0f }, //3
        { 0.0f,-0.4f, 1.0f, 0.0f, 0.0f,1.0f, 0.5f, 0.3f }, //2
		{ 0.0f, 0.0f, 0.7f, 0.0f, 0.0f,1.0f, 0.5f, 0.5f },
        { 0.8f,-1.0f, 1.0f, 0.0f, 0.0f,1.0f, 0.9f, 0.0f }, //3
        { 0.4f,-0.2f, 1.0f, 0.0f, 0.0f,1.0f, 0.7f, 0.4f }, //5
		{ 0.0f, 0.0f, 0.7f, 0.0f, 0.0f,1.0f, 0.5f, 0.5f },
        { 1.0f, 0.5f, 1.0f, 0.0f, 0.0f,1.0f, 1.0f, 0.75f}, //10
        { 0.4f,-0.2f, 1.0f, 0.0f, 0.0f,1.0f, 0.7f, 0.4f }, //5
		{ 0.0f, 0.0f, 0.7f, 0.0f, 0.0f,1.0f, 0.5f, 0.5f },
        { 1.0f, 0.5f, 1.0f, 0.0f, 0.0f,1.0f, 1.0f, 0.75f}, //10
        { 0.2f, 0.3f, 1.0f, 0.0f, 0.0f,1.0f, 0.6f, 0.65f}, //9
		{ 0.0f, 0.0f, 0.7f, 0.0f, 0.0f,1.0f, 0.5f, 0.5f },
        { 0.0f, 1.0f, 1.0f, 0.0f, 0.0f,1.0f, 0.5f, 1.0f }, //8
        { 0.2f, 0.3f, 1.0f, 0.0f, 0.0f,1.0f, 0.6f, 0.65f}, //9
		{ 0.0f, 0.0f, 0.7f, 0.0f, 0.0f,1.0f, 0.5f, 0.5f },
        { 0.0f, 1.0f, 1.0f, 0.0f, 0.0f,1.0f, 0.5f, 1.0f }, //8
		{-0.2f, 0.3f, 1.0f, 0.0f, 0.0f,1.0f, 0.4f, 0.65f}, //7
		{ 0.0f, 0.0f, 0.7f, 0.0f, 0.0f,1.0f, 0.5f, 0.5f },
        {-1.0f, 0.5f, 1.0f, 0.0f, 0.0f,1.0f, 0.0f, 0.75f}, //6
		{-0.2f, 0.3f, 1.0f, 0.0f, 0.0f,1.0f, 0.4f, 0.65f}, //7
		{ 0.0f, 0.0f, 0.7f, 0.0f, 0.0f,1.0f, 0.5f, 0.5f },
		{-1.0f, 0.5f, 1.0f, 0.0f, 0.0f,1.0f, 0.0f, 0.75f}, //6
		{-0.4f,-0.2f, 1.0f, 0.0f, 0.0f,1.0f, 0.3f, 0.4f }, //4
		{ 0.0f, 0.0f, 0.7f, 0.0f, 0.0f,1.0f, 0.5f, 0.5f },*/

        { 0.2f,-3.0f, 80.0f, 0.0f, 0.0f, -1.0f, 0.1f, 0.0f }, //1
		{ 1.0f,-2.4f, 80.0f, 0.0f, 0.0f, -1.0f, 0.5f, 0.3f }, //2
		{ 1.0f,-2.0f, 80.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
		{ 0.2f,-3.0f, 80.0f, 0.0f, 0.0f, -1.0f, 0.1f, 0.0f }, //1
		{ 0.6f,-2.2f, 80.0f, 0.0f, 0.0f, -1.0f, 0.3f, 0.4f }, //4
		{ 1.0f, -2.0f,80.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
        { 1.8f,-3.0f, 80.0f, 0.0f, 0.0f, -1.0f, 0.9f, 0.0f }, //3
        { 1.0f,-2.4f, 80.0f, 0.0f, 0.0f, -1.0f, 0.5f, 0.3f }, //2
		{ 1.0f, -2.0f, 80.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
        { 1.8f,-3.0f, 80.0f, 0.0f, 0.0f, -1.0f, 0.9f, 0.0f }, //3
        { 1.4f,-2.2f, 80.0f, 0.0f, 0.0f, -1.0f, 0.7f, 0.4f }, //5
		{ 1.0f, -2.0f, 80.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
        { 2.0f, -1.5f, 80.0f, 0.0f, 0.0f, -1.0f, 1.0f, 0.75f}, //10
        { 1.4f,-2.2f, 80.0f, 0.0f, 0.0f, -1.0f, 0.7f, 0.4f }, //5
		{ 1.0f, -2.0f, 80.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
        { 2.0f, -1.5f, 80.0f, 0.0f, 0.0f, -1.0f, 1.0f, 0.75f}, //10
        { 1.2f, -1.7f, 80.0f, 0.0f, 0.0f, -1.0f, 0.6f, 0.65f}, //9
		{ 1.0f, -2.0f, 80.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
        { 1.0f, -1.0f, 80.0f, 0.0f, 0.0f, -1.0f, 0.5f, 1.0f }, //8
        { 1.2f, -1.7f, 80.0f, 0.0f, 0.0f, -1.0f, 0.6f, 0.65f}, //9
		{ 1.0f, -2.0f, 80.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
        { 1.0f, -1.0f, 80.0f, 0.0f, 0.0f, -1.0f, 0.5f, 1.0f }, //8
		{ 0.8f, -1.7f, 80.0f, 0.0f, 0.0f, -1.0f, 0.4f, 0.65f}, //7
		{ 1.0f, -2.0f, 80.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
        { 0.0f, -1.5f, 80.0f, 0.0f, 0.0f, -1.0f, 0.0f, 0.75f}, //6
		{ 0.8f, -1.7f, 80.0f, 0.0f, 0.0f, -1.0f, 0.4f, 0.65f}, //7
		{ 1.0f, -2.0f, 80.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
		{ 0.0f, -1.5f, 80.0f, 0.0f, 0.0f, -1.0f, 0.0f, 0.75f}, //6
		{ 0.6f,-2.2f, 80.0f, 0.0f, 0.0f, -1.0f, 0.3f, 0.4f }, //4
		{ 1.0f, -2.0f, 80.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },


		{-0.8f,-1.0f, 100.0f, 0.0f, 0.0f, -1.0f, 0.1f, 0.0f }, //1
		{ 0.0f,-0.4f, 100.0f, 0.0f, 0.0f, -1.0f, 0.5f, 0.3f }, //2
		{ 0.0f, 0.0f, 100.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
		{-0.8f,-1.0f, 100.0f, 0.0f, 0.0f, -1.0f, 0.1f, 0.0f }, //1
		{-0.4f,-0.2f, 100.0f, 0.0f, 0.0f, -1.0f, 0.3f, 0.4f }, //4
		{ 0.0f, 0.0f, 100.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
        { 0.8f,-1.0f, 100.0f, 0.0f, 0.0f, -1.0f, 0.9f, 0.0f }, //3
        { 0.0f,-0.4f, 100.0f, 0.0f, 0.0f, -1.0f, 0.5f, 0.3f }, //2
		{ 0.0f, 0.0f, 100.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
        { 0.8f,-1.0f, 100.0f, 0.0f, 0.0f, -1.0f, 0.9f, 0.0f }, //3
        { 0.4f,-0.2f, 100.0f, 0.0f, 0.0f, -1.0f, 0.7f, 0.4f }, //5
		{ 0.0f, 0.0f, 100.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
        { 1.0f, 0.5f, 100.0f, 0.0f, 0.0f, -1.0f, 1.0f, 0.75f}, //10
        { 0.4f,-0.2f, 100.0f, 0.0f, 0.0f, -1.0f, 0.7f, 0.4f }, //5
		{ 0.0f, 0.0f, 100.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
        { 1.0f, 0.5f, 100.0f, 0.0f, 0.0f, -1.0f, 1.0f, 0.75f}, //10
        { 0.2f, 0.3f, 100.0f, 0.0f, 0.0f, -1.0f, 0.6f, 0.65f}, //9
		{ 0.0f, 0.0f, 100.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
        { 0.0f, 1.0f, 100.0f, 0.0f, 0.0f, -1.0f, 0.5f, 1.0f }, //8
        { 0.2f, 0.3f, 100.0f, 0.0f, 0.0f, -1.0f, 0.6f, 0.65f}, //9
		{ 0.0f, 0.0f, 100.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
        { 0.0f, 1.0f, 100.0f, 0.0f, 0.0f, -1.0f, 0.5f, 1.0f }, //8
		{-0.2f, 0.3f, 100.0f, 0.0f, 0.0f, -1.0f, 0.4f, 0.65f}, //7
		{ 0.0f, 0.0f, 100.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
        {-1.0f, 0.5f, 100.0f, 0.0f, 0.0f, -1.0f, 0.0f, 0.75f}, //6
		{-0.2f, 0.3f, 100.0f, 0.0f, 0.0f, -1.0f, 0.4f, 0.65f}, //7
		{ 0.0f, 0.0f, 100.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f },
		{-1.0f, 0.5f, 100.0f, 0.0f, 0.0f, -1.0f, 0.0f, 0.75f}, //6
		{-0.4f,-0.2f, 100.0f, 0.0f, 0.0f, -1.0f, 0.3f, 0.4f }, //4
		{ 0.0f, 0.0f, 100.3f, 0.0f, 0.0f, -1.0f, 0.5f, 0.5f }
/*		{-0.8f,-1.0f, 1.1f, 0.0f, 0.0f, 1.0f, 0.1f, 0.0f }, //1
        {-0.4f,-0.2f, 1.1f, 0.0f, 0.0f, 1.0f, 0.3f, 0.4f }, //4
        { 0.0f,-0.4f, 1.1f, 0.0f, 0.0f, 1.0f, 0.5f, 0.3f }, //2
        { 0.0f,-0.4f, 1.1f, 0.0f, 0.0f, 1.0f, 0.5f, 0.3f }, //2
        { 0.4f,-0.2f, 1.1f, 0.0f, 0.0f, 1.0f, 0.7f, 0.4f }, //5
        { 0.8f,-1.0f, 1.1f, 0.0f, 0.0f, 1.0f, 0.9f, 0.0f }, //3
        {-1.0f, 0.5f, 1.1f, 0.0f, 0.0f, 1.0f, 0.0f, 0.75f}, //6
        {-0.4f,-0.2f, 1.1f, 0.0f, 0.0f, 1.0f, 0.3f, 0.4f }, //4
        {-0.2f, 0.3f, 1.1f, 0.0f, 0.0f, 1.0f, 0.4f, 0.65f}, //7
        { 1.0f, 0.5f, 1.1f, 0.0f, 0.0f, 1.0f, 1.0f, 0.75f}, //10
        { 0.4f,-0.2f, 1.1f, 0.0f, 0.0f, 1.0f, 0.7f, 0.4f }, //5
        { 0.2f, 0.3f, 1.1f, 0.0f, 0.0f, 1.0f, 0.6f, 0.65f}, //9
		{-0.2f, 0.3f, 1.1f, 0.0f, 0.0f, 1.0f, 0.4f, 0.65f}, //7
        { 0.2f, 0.3f, 1.1f, 0.0f, 0.0f, 1.0f, 0.6f, 0.65f}, //9
        { 0.0f, 1.0f, 1.1f, 0.0f, 0.0f, 1.0f, 0.5f, 1.0f }, //8
        {-0.4f,-0.2f, 1.1f, 0.0f, 0.0f, 1.0f, 0.3f, 0.4f }, //4
        { 0.0f,-0.4f, 1.1f, 0.0f, 0.0f, 1.0f, 0.5f, 0.3f }, //2
        {-0.2f, 0.3f, 1.1f, 0.0f, 0.0f, 1.0f, 0.4f, 0.65f}, //7
        { 0.0f,-0.4f, 1.1f, 0.0f, 0.0f, 1.0f, 0.5f, 0.3f }, //2
        {-0.2f, 0.3f, 1.1f, 0.0f, 0.0f, 1.0f, 0.4f, 0.65f}, //7
        { 0.2f, 0.3f, 1.1f, 0.0f, 0.0f, 1.0f, 0.6f, 0.65f}, //9
        { 0.0f,-0.4f, 1.1f, 0.0f, 0.0f, 1.0f, 0.5f, 0.3f }, //2
        { 0.2f, 0.3f, 1.1f, 0.0f, 0.0f, 1.0f, 0.6f, 0.65f}, //9
        { 0.4f,-0.2f, 1.1f, 0.0f, 0.0f, 1.0f, 0.7f, 0.4f }  //5
		*/
    };

	/*	{	
		{-1.0f,-1.0f, 1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f, }, //FRONT
        { 1.0f,-1.0f, 1.0f, 0.0f, 0.0f,-1.0f, 1.0f, 0.0f, },
        { 1.0f, 1.0f, 1.0f, 0.0f, 0.0f,-1.0f, 1.0f, 1.0f, },
        {-1.0f,-1.0f, 1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f, },
        {-1.0f, 1.0f, 1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 1.0f, },
        { 1.0f, 1.0f, 1.0f, 0.0f, 0.0f,-1.0f, 1.0f, 1.0f, },
        {-1.0f,-1.0f, 1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f, },
        {-1.0f, 1.0f, 1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 1.0f, },
        { 1.0f, 1.0f, 1.0f, 0.0f, 0.0f,-1.0f, 1.0f, 1.0f, }
    };
 	*/



    // Create the vertex buffer.
    if( FAILED( hr = m_pd3dDevice->CreateVertexBuffer( 60*sizeof(CUSTOMVERTEX),
                                                  0, D3DFVF_CUSTOMVERTEX,
                                                  D3DPOOL_DEFAULT, &m_pVB ) ) )
    {
        *m_logfile << LOCATION << "Could not create vertex buffer";
        return E_FAIL;
    }

    // Fill the vertex buffer.
    VOID* pVertices;
    if( FAILED( m_pVB->Lock( 0, sizeof(m_Vertices), (BYTE**)&pVertices, 0 ) ) )
        return E_FAIL;
    memcpy( pVertices, m_Vertices, sizeof(m_Vertices) );
    m_pVB->Unlock();



////////////////////////////////////////////////////////////////////////////////

    // Initialize three vertices for rendering a triangle
    CUSTOMVERTEX m_Vertices2[] =
    {	
		{-100.0f,-100.0f, 200.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f, }, //FRONT
        { 100.0f,-100.0f, 200.0f, 0.0f, 0.0f,-1.0f, 1.0f, 0.0f, },
        { 100.0f, 100.0f, 200.0f, 0.0f, 0.0f,-1.0f, 1.0f, 1.0f, },
        {-100.0f,-100.0f, 200.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f, },
        {-100.0f, 100.0f, 200.0f, 0.0f, 0.0f,-1.0f, 0.0f, 1.0f, },
        { 100.0f, 100.0f, 200.0f, 0.0f, 0.0f,-1.0f, 1.0f, 1.0f, },
    };

    // Create the vertex buffer.
    if( FAILED( hr = m_pd3dDevice->CreateVertexBuffer( 6*sizeof(CUSTOMVERTEX),
                                                  0, D3DFVF_CUSTOMVERTEX,
                                                  D3DPOOL_DEFAULT, &m_pVB_background ) ) )
    {
        *m_logfile << LOCATION << "Could not create vertex buffer environment";
        return E_FAIL;
    }

    // Fill the vertex buffer.
    VOID* pVertices2;
    if( FAILED( m_pVB_background->Lock( 0, sizeof(m_Vertices2), (BYTE**)&pVertices2, 0 ) ) )
        return E_FAIL;
    memcpy( pVertices2, m_Vertices2, sizeof(m_Vertices2) );
    m_pVB_background->Unlock();

////////////////////////////////////////////////////////////////////////////////





	// Create the shader declaration.
	DWORD dwDecl[] =
	{
		D3DVSD_STREAM(0),
		D3DVSD_REG(D3DVSDE_POSITION,  D3DVSDT_FLOAT3),
		D3DVSD_REG(D3DVSDE_NORMAL, D3DVSDT_FLOAT3),
		D3DVSD_REG(D3DVSDE_TEXCOORD1, D3DVSDT_FLOAT2),
		D3DVSD_END()
	};

    // Create the vertex shaders.
    TCHAR        strPath[512];                      // location of the shader file
    LPD3DXBUFFER pCode;                             // assembled shader code
	lstrcpy( strPath, VERTEX_SHADER_LOCATION );
	if( FAILED(D3DXAssembleShaderFromFile( strPath, 0, NULL, &pCode, NULL )) ) {
		*m_logfile << LOCATION << "did not find vertex shader in" << strPath;	
		m_logfile->flush();
	}
	else
	{
		if (FAILED(m_pd3dDevice->CreateVertexShader( dwDecl, (DWORD*)pCode->GetBufferPointer(), &m_hLightningVertexShader, 0 )))
			*m_logfile << LOCATION << "Failed to create the vertex shader";
		pCode->Release();
	}

	// Create the pixel shader
	lstrcpy( strPath, PIXEL_SHADER_LOCATION );
	if( FAILED(D3DXAssembleShaderFromFile( strPath, 0, NULL, &pCode, NULL )) ) {  // assemble shader code
		*m_logfile << LOCATION << "did not find vertex shader in" << strPath;	
		m_logfile->flush();
	}
	else {
		if (FAILED(m_pd3dDevice->CreatePixelShader( (DWORD*)pCode->GetBufferPointer(), &m_hLightningPixelShader )))
			*m_logfile << LOCATION << "Failed to create the pixel shader";
		pCode->Release();
	}	

	m_logfile->flush();

	m_lVidPitch = (2*m_iImageWidth * 3 + 3) & ~(3);


    return S_OK;
}


//-----------------------------------------------------------------------------
// Name: SetupMatrices()
// Desc: Sets up the world, view, and projection transform matrices.
//-----------------------------------------------------------------------------
HRESULT CVirtualStream::SetupMatrices(float displacement, double time)
{
	HRESULT hr;
    // For our world matrix, we will just leave it as the identity
    D3DXMATRIX matWorld;
    D3DXMatrixIdentity( &matWorld );
    //D3DXMatrixRotationZ( &matWorld, timeGetTime()/10000.0f );

    // Set up our view matrix. A view matrix can be defined given an eye point,
    // a point to lookat, and a direction for which way is up. Here, we set the
    // eye five units back along the z-axis and up three units, look at the
    // origin, and define "up" to be in the y-direction.
    D3DXMATRIX matView;
/*
	D3DXVECTOR3 posVec = D3DXVECTOR3( 4.0f*sinf(float(time/100000)+0.2f*displacement), 0.0f, -6.0f*cosf(float(time/100000) + 0.2f*displacement));
	D3DXVECTOR3 lookAtVec = D3DXVECTOR3( 100.0f*sinf(float(time/100000) + PI), 0.0f, -100.0f*cosf(float(time/100000)+PI));
*/    
	D3DXVECTOR3 posVec = D3DXVECTOR3( displacement, 0.0f, -75.0f*sinf(float(time/10000)));
	D3DXVECTOR3 lookAtVec = D3DXVECTOR3( displacement, 0.0f, 100.0f);
	D3DXMatrixLookAtLH( &matView, &posVec,
                                  &lookAtVec,
                                  &D3DXVECTOR3( 0.0f, 1.0f, 0.0f ) );

    // For the projection matrix, we set up a perspective transform (which
    // transforms geometry from 3D view space to 2D viewport space, with
    // a perspective divide making objects smaller in the distance). To build
    // a perpsective transform, we need the field of view (1/4 pi is common),
    // the aspect ratio, and the near and far clipping planes (which define at
    // what distances geometry should be no longer be rendered).
    D3DXMATRIX matProj;
    D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/4, 1.0f, 0.0f, 100.0f );

	D3DXMATRIX mat;
	D3DXMatrixMultiply( &mat, &matView, &matProj );
	D3DXMatrixTranspose( &mat, &mat );
    if (FAILED(hr = m_pd3dDevice->SetVertexShaderConstant(0, &mat, 4)))                                      
    {
        *m_logfile << LOCATION << "Could not set Vertex Shader total transform matrix!";
        return hr;
    }

	if (FAILED(hr = m_pd3dDevice->SetVertexShaderConstant(4, &matView, 4)))                                      
    {
        *m_logfile << LOCATION << "Could not set Vertex Shader world-view transform matrix!";
        return hr;
    }
	
	D3DXVECTOR4 sightVec = lookAtVec - posVec;
	if (FAILED(hr = m_pd3dDevice->SetVertexShaderConstant(8, &sightVec, 1)))                                      
    {
        *m_logfile << LOCATION << "Could not set Vertex Shader sight vector!r";
        return hr;
    }

	D3DXMATRIX lightMatrix1 = D3DXMATRIX( 1.0f, 1.0f, 1.0f, 0.1f,
										  0.5f, 0.1f, 0.1f, 0.5f,
										  0.5f, 0.8f, 0.1f, 0.5f,
										  0.5f, 0.2f, 0.1f, 0.5f);
	if (FAILED(hr = m_pd3dDevice->SetVertexShaderConstant(9, &lightMatrix1, 4)))                                      
    {
        *m_logfile << LOCATION << "Could not set Vertex Shader light vectors!";
        return hr;
    }
	
	D3DXMATRIX lightMatrix2 = D3DXMATRIX( 1.0f, -1.0f, -1.0f, 0.5f,
										  0.2f, 0.2f, 1.0f, 0.5f,
										  0.2f, 0.2f, 1.0f, 0.5f,
										  0.2f, 0.2f, 0.5f, 0.5f);
	if (FAILED(hr = m_pd3dDevice->SetVertexShaderConstant(13, &lightMatrix2, 4)))                                      
    {
        *m_logfile << LOCATION << "Could not set Vertex Shader light vectors!";
        return hr;
    }

	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Draws the scene
//-----------------------------------------------------------------------------
HRESULT CVirtualStream::Render(LPDIRECT3DSURFACE8 surf, float displacement, double time)
{
	HRESULT hr;

	if(SUCCEEDED(m_pRenderToSurface->BeginScene(surf, NULL)))
    {


		// Begin the scene
		m_pd3dDevice->BeginScene();

		// Clear the suface
		if (FAILED(m_pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET, D3DCOLOR_XRGB(0,0,255), 1.0f, 0)))
		{
			*m_logfile << LOCATION << "Could not clear the render target";
			return E_FAIL;
		}
		if (FAILED(m_pd3dDevice->SetVertexShader( hr = m_hLightningVertexShader )))
		{
			*m_logfile << LOCATION << "Could not set vertex shader hr = "<< (hr&0xFFFF);
			return E_FAIL;
		}
		if (FAILED(m_pd3dDevice->SetPixelShader( m_hLightningPixelShader )))
		{
			*m_logfile << LOCATION << "Could not set pixel shader";
			return E_FAIL;
		}



		// Setup the world, view, and projection matrices
		if (FAILED(SetupMatrices(0.0f,0.0)))
		{
			*m_logfile << LOCATION << "Could not setup matrices";
			return E_FAIL;
		}
		// Set texture, vertex shader, pixel shader and vertex buffer
		if (FAILED(m_pd3dDevice->SetTexture( 0, m_pTexture_environment )))
		{
			*m_logfile << LOCATION << "Could not set texture";
			return E_FAIL;
		}
		if (FAILED(m_pd3dDevice->SetStreamSource( 0, m_pVB_background, sizeof(CUSTOMVERTEX) )))
		{
			*m_logfile << LOCATION << "Could not set stream source";
			return E_FAIL;
		}
		if (FAILED(hr = m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 2 )))
		{
			*m_logfile << LOCATION << "Could not draw primitive hr = " << (hr&0xFFFF);
			return E_FAIL;
		}





		// Setup the world, view, and projection matrices
		if (FAILED(SetupMatrices(displacement,time)))
		{
			*m_logfile << LOCATION << "Could not setup matrices";
			return E_FAIL;
		}
			
		// Set texture, vertex shader, pixel shader and vertex buffer
		if (FAILED(m_pd3dDevice->SetTexture( 0, m_pTexture )))
		{
			*m_logfile << LOCATION << "Could not set texture";
			return E_FAIL;
		}
		if (FAILED(m_pd3dDevice->SetStreamSource( 0, m_pVB, sizeof(CUSTOMVERTEX) )))
		{
			*m_logfile << LOCATION << "Could not set stream source";
			return E_FAIL;
		}
		if (FAILED(hr = m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 20 )))
		{
			*m_logfile << LOCATION << "Could not draw primitive hr = " << (hr&0xFFFF);
			return E_FAIL;
		}

		m_pRenderToSurface->EndScene();

	return S_OK;
    }
    else
	{
        *m_logfile << LOCATION << "Could not render to surface left";
        return E_FAIL;
    }


}

//-----------------------------------------------------------------------------
// CVirtualStream FillBuffer
// Required
// Plots a virtual image into the supplied video buffer
//-----------------------------------------------------------------------------
HRESULT CVirtualStream::FillBuffer(IMediaSample *pSample )
{
    BYTE  *pBmpBuffer, *pTxtBufferLeft, *pTxtBufferRight;   // Bitmap buffer, texture buffer
    LONG  lTxtPitch;										// Pitch of bitmap, texture
    long lDataLen;
 
	// Get the video bitmap buffer
    pSample->GetPointer( &pBmpBuffer );
    lDataLen = pSample->GetSize();

    // If true then we clear the output buffer and don't attempt to
    // erase a previous drawing - this will be the case
    // when we start running as the buffer will be full of rubbish
    if( m_bZeroMemory ) {
        ZeroMemory( pBmpBuffer, lDataLen );
    }

	//LPDIRECT3DTEXTURE8 p_D3DTexture;
	{
		CAutoLock cAutoLockShared(&m_cSharedState);
		
		//Draw the texture
		double time = timeGetTime();
		if (FAILED(Render(m_pRenderTargetSurfaceLeft,-0.5f,time)))
			*m_logfile << LOCATION << "Failed to render to surface left";
		if (FAILED(Render(m_pRenderTargetSurfaceRight,0.5f,time)))
			*m_logfile << LOCATION << "Failed to render to surface right";

		// Lock the Textures
		D3DLOCKED_RECT d3dlrleft;
		if (FAILED(m_pRenderTargetSurfaceLeft->LockRect(&d3dlrleft, 0, 0)))
			return E_FAIL;
		D3DLOCKED_RECT d3dlrright;
		if (FAILED(m_pRenderTargetSurfaceRight->LockRect(&d3dlrright, 0, 0)))
			return E_FAIL;
		// Get the texture buffer & pitch
		pTxtBufferLeft = static_cast<byte *>(d3dlrleft.pBits);
		pTxtBufferRight = static_cast<byte *>(d3dlrright.pBits);
		lTxtPitch = d3dlrleft.Pitch;


		for(int y = 0; y < m_iImageHeight; y++ ) {
			BYTE *pBmpBufferOld  = pBmpBuffer;
			BYTE *pTxtBufferOldLeft = pTxtBufferLeft;  
			BYTE *pTxtBufferOldRight = pTxtBufferRight;  
			for (int x = 0; x < m_iImageWidth; x++) {
				pBmpBuffer[0] = pTxtBufferLeft[0];
				pBmpBuffer[1] = pTxtBufferLeft[1];
				pBmpBuffer[2] = pTxtBufferLeft[2];
				pTxtBufferLeft += 4;
				pBmpBuffer += 3;
				}
			for (x = 0; x < m_iImageWidth; x++) {
				pBmpBuffer[0] = pTxtBufferRight[0];
				pBmpBuffer[1] = pTxtBufferRight[1];
				pBmpBuffer[2] = pTxtBufferRight[2];
				pTxtBufferRight += 4;
				pBmpBuffer += 3;
				}
			pBmpBuffer = pBmpBufferOld + m_lVidPitch;
			pTxtBufferLeft = pTxtBufferOldLeft + lTxtPitch;
			pTxtBufferRight = pTxtBufferOldRight + lTxtPitch;
		}

        // The current time is the sample's start
        CRefTime rtStart = m_rtSampleTime;

        // Increment to find the finish time
        m_rtSampleTime += (LONG)m_iRepeatTime;

        pSample->SetTime((REFERENCE_TIME *) &rtStart,(REFERENCE_TIME *) &m_rtSampleTime);
    }
      

    // Unlock the Texture
    if (FAILED(m_pRenderTargetSurfaceLeft->UnlockRect()))
        return E_FAIL;
	// Unlock the Texture
    if (FAILED(m_pRenderTargetSurfaceRight->UnlockRect()))
        return E_FAIL;
    m_bZeroMemory = FALSE;
    pSample->SetSyncPoint(TRUE);

    return S_OK;
}


//-----------------------------------------------------------------------------
// CVirtualStream notify
//
// Alter the repeat rate according to quality management messages sent from
// the downstream filter (often the renderer).  Wind it up or down according
// to the flooding level - also skip forward if we are notified of Late-ness
//-----------------------------------------------------------------------------
STDMETHODIMP CVirtualStream::Notify(IBaseFilter * pSender, Quality q)
{
    // Adjust the repeat rate.
    if (q.Proportion<=0) {
        m_iRepeatTime = 10000;        // We don't go slower than 1 per second
    } else {
        m_iRepeatTime = m_iRepeatTime*10000/q.Proportion;
        if (m_iRepeatTime>10000) {
            m_iRepeatTime = 10000;    // We don't go slower than 1 per second
        } else if (m_iRepeatTime<10) {
            m_iRepeatTime = 10;      // We don't go faster than 100/sec
        }
    }

    // skip forwards
    if (q.Late > 0) {
        m_rtSampleTime += q.Late;
    }
    return NOERROR;

} // Notify


//-----------------------------------------------------------------------------
// CVirtualStream GetMediaType
// Required
//-----------------------------------------------------------------------------
HRESULT CVirtualStream::GetMediaType(int iPosition, CMediaType *pmt)
{
    CAutoLock cAutoLock(m_pFilter->pStateLock());
    if (iPosition < 0) {
        return E_INVALIDARG;
    }

    // Have we run off the end of types

    if (iPosition > 4) {
        return VFW_S_NO_MORE_ITEMS;
    }

    VIDEOINFO *pvi = (VIDEOINFO *) pmt->AllocFormatBuffer(sizeof(VIDEOINFO));
    if (NULL == pvi) {
	return(E_OUTOFMEMORY);
    }
    ZeroMemory(pvi, sizeof(VIDEOINFO));

  
    pvi->bmiHeader.biCompression = BI_RGB;
    pvi->bmiHeader.biBitCount    = 24;
     

    // (Adjust the parameters common to all formats...)

    pvi->bmiHeader.biSize		= sizeof(BITMAPINFOHEADER);
    pvi->bmiHeader.biWidth		= 2*m_iImageWidth;
    pvi->bmiHeader.biHeight		= m_iImageHeight;
    pvi->bmiHeader.biPlanes		= 1;
    pvi->bmiHeader.biSizeImage		= GetBitmapSize(&pvi->bmiHeader);
    pvi->bmiHeader.biClrImportant	= 0;

//    SetRectEmpty(&(pvi->rcSource));	// we want the whole image area rendered.
	pvi->rcSource.top=0;
	pvi->rcSource.left=0;
	pvi->rcSource.right=2*m_iImageWidth;
	pvi->rcSource.bottom=m_iImageHeight;
    SetRectEmpty(&(pvi->rcTarget));	// no particular destination rectangle

    pmt->SetType(&MEDIATYPE_Video);
    pmt->SetFormatType(&FORMAT_VideoInfo);
    pmt->SetTemporalCompression(FALSE);

    // Work out the GUID for the subtype from the header info.
    const GUID SubTypeGUID = GetBitmapSubtype(&pvi->bmiHeader);
    pmt->SetSubtype(&SubTypeGUID);
    pmt->SetSampleSize(pvi->bmiHeader.biSizeImage);

    return NOERROR;

} // GetMediaType

//-----------------------------------------------------------------------------
// CVirtualStream CheckMediaType
// Required
//-----------------------------------------------------------------------------
// We will accept 24 bit video formats, in any
// image size that gives room to bounce.
// Returns E_INVALIDARG if the mediatype is not acceptable
//
HRESULT CVirtualStream::CheckMediaType(const CMediaType *pMediaType)
{
    CAutoLock cAutoLock(m_pFilter->pStateLock());

    if ((*(pMediaType->Type()) != MEDIATYPE_Video)	// we only output video!
	    || !(pMediaType->IsFixedSize()) ) {		// ...in fixed size samples
                return E_INVALIDARG;
    }

    // Check for the subtypes we support
    const GUID *SubType = pMediaType->Subtype();
    if (*SubType != MEDIASUBTYPE_RGB24)
	{
                return E_INVALIDARG;
    }

    // Get the format area of the media type
    VIDEOINFO *pvi = (VIDEOINFO *) pMediaType->Format();

    if (pvi == NULL)
		return E_INVALIDARG;

	if (pvi->bmiHeader.biWidth!=2*m_iImageWidth || pvi->bmiHeader.biHeight!=m_iImageHeight)
		return E_INVALIDARG;

    return S_OK;  // This format is acceptable.

} // CheckMediaType

//-----------------------------------------------------------------------------
// CVirtualStream DecideBufferSize
//-----------------------------------------------------------------------------
// This will always be called after the format has been sucessfully
// negotiated. So we have a look at m_mt to see what size image we agreed.
// Then we can ask for buffers of the correct size to contain them.
//
HRESULT CVirtualStream::DecideBufferSize(IMemAllocator *pAlloc,ALLOCATOR_PROPERTIES *pProperties)
{
    CAutoLock cAutoLock(m_pFilter->pStateLock());
    ASSERT(pAlloc);
    ASSERT(pProperties); 
    HRESULT hr = NOERROR;

    VIDEOINFO *pvi = (VIDEOINFO *) m_mt.Format();
    pProperties->cBuffers = 1;
    pProperties->cbBuffer = pvi->bmiHeader.biSizeImage;

    ASSERT(pProperties->cbBuffer);

    // Ask the allocator to reserve us some sample memory, NOTE the function
    // can succeed (that is return NOERROR) but still not have allocated the
    // memory that we requested, so we must check we got whatever we wanted

    ALLOCATOR_PROPERTIES Actual;
    hr = pAlloc->SetProperties(pProperties,&Actual);
    if (FAILED(hr)) {
        return hr;
    }

    // Is this allocator unsuitable

    if (Actual.cbBuffer < pProperties->cbBuffer) {
        return E_FAIL;
    }

    // Make sure that we have only 1 buffer (we erase the bal in the
    // old buffer to save having to zero a 200k+ buffer every time
    // we draw a frame)

    ASSERT( Actual.cBuffers == 1 );
    return NOERROR;

} // DecideBufferSize


//-----------------------------------------------------------------------------
// CVirtualStream SetMediaType
//-----------------------------------------------------------------------------
// Called when a media type is agreed between filters
//
HRESULT CVirtualStream::SetMediaType(const CMediaType *pMediaType)
{
    CAutoLock cAutoLock(m_pFilter->pStateLock());

    // Pass the call up to my base class

    HRESULT hr = CSourceStream::SetMediaType(pMediaType);
    if (SUCCEEDED(hr)) {

        VIDEOINFO * pvi = (VIDEOINFO *) m_mt.Format();
	//	if (pvi->bmiHeader.biWidth==2*m_iImageWidth)
			return NOERROR;
	//	else
	//		return E_INVALIDARG;
    }  

    return hr;

} // SetMediaType


//-----------------------------------------------------------------------------
// CVirtualStream OnThreadCreate
//-----------------------------------------------------------------------------
// As we go active reset the stream time to zero
//
HRESULT CVirtualStream::OnThreadCreate()
{
    CAutoLock cAutoLockShared(&m_cSharedState);
    m_rtSampleTime = 0;

    // we need to also reset the repeat time in case the system
    // clock is turned off after m_iRepeatTime gets very big
    m_iRepeatTime = m_iDefaultRepeatTime;

    // Zero the output buffer on the first frame.
    m_bZeroMemory = TRUE;

    return NOERROR;

} // OnThreadCreate

//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: The window's message handler
//-----------------------------------------------------------------------------
LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg )
    {
        case WM_DESTROY:
            PostQuitMessage( 0 );
            return 0;
    }

    return DefWindowProc( hWnd, msg, wParam, lParam );
}