//------------------------------------------------------------------------------
// File: VirtualSOurce.h
//------------------------------------------------------------------------------
#include <d3dx8.h>
#include <fstream.h>

#define LOGFILE_LOCATION "e:\\common\\virtualsource_logfile.txt"
#define TEXTURE_LOCATION "e:\\common\\virtualsource\\banana.bmp"
#define ENVIRONMENT_LOCATION "e:\\common\\virtualsource\\env3.bmp"
#define VERTEX_SHADER_LOCATION "e:\\common\\VirtualSource\\LightningVertexShader.txt"
#define PIXEL_SHADER_LOCATION  "e:\\common\\VirtualSource\\LightningPixelShader.txt"

// This define is used for writing out the current file and line when logging
#define LOCATION "\n" << __FILE__ << " : Line "<< __LINE__ << "\n"

// Define the sizeof the video created
#define VID_WIDTH 256
#define VID_HEIGHT 256
#define PI 3.1415926535f

// Is this GUID taken?
DEFINE_GUID(CLSID_CVirtualSource,
0xfd501041, 0x8ebe, 0x11ce, 0x81, 0x83, 0x00, 0xaa, 0x00, 0x57, 0x7d, 0xa1);

//------------------------------------------------------------------------------
// Forward Declarations
//------------------------------------------------------------------------------
// The class managing the output pin
class CVirtualStream;

//------------------------------------------------------------------------------
// Class CVirtualSource
//
// This is the main class for the virtual video filter. It inherits from
// CSource, the DirectShow base class for source filters.
//------------------------------------------------------------------------------
class CVirtualSource :	public CSource
						//,public ISpecifyPropertyPages
{
public:

    // The only allowed way to create Bouncing balls!
    static CUnknown * WINAPI CreateInstance(LPUNKNOWN lpunk, HRESULT *phr);

/*	// Reveals IContrast & ISpecifyPropertyPages
	STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void ** ppv);
	
	DECLARE_IUNKNOWN;

	// ISpecifyPropertyPages method
    STDMETHODIMP GetPages(CAUUID *pPages);*/

private:

    // It is only allowed to to create these objects with CreateInstance
    CVirtualSource(LPUNKNOWN lpunk, HRESULT *phr);

}; // CBouncingBall


//------------------------------------------------------------------------------
// Class CVirtualStream
//
// This class implements the stream which is used to output the virtual video
// data from the source filter. It inherits from DirectShows's base
// CSourceStream class.
//------------------------------------------------------------------------------
class CVirtualStream : public CSourceStream
{

public:

    CVirtualStream(HRESULT *phr, CVirtualSource *pParent, LPCWSTR pPinName);
    ~CVirtualStream();

    // plots video into the supplied video frame
    HRESULT FillBuffer(IMediaSample *pms);

    // Ask for buffers of the size appropriate to the agreed media type
    HRESULT DecideBufferSize(IMemAllocator *pIMemAlloc,
                             ALLOCATOR_PROPERTIES *pProperties);

    // Set the agreed media type, and set up parameters
    HRESULT SetMediaType(const CMediaType *pMediaType);

    HRESULT CheckMediaType(const CMediaType *pMediaType);
    HRESULT GetMediaType(int iPosition, CMediaType *pmt);

    // Resets the stream time to zero
    HRESULT OnThreadCreate(void);

    // Quality control notifications sent to us
    STDMETHODIMP Notify(IBaseFilter * pSender, Quality q);

private:

	// D3D help methods
	HRESULT InitD3D( HWND hWnd );
	HRESULT InitGeometry();
	HRESULT SetupMatrices(float displacement, double time);
	HRESULT Render(LPDIRECT3DSURFACE8 surf, float displacement, double time);

	// D3D objects
	LPDIRECT3D8				m_pD3D;	
	LPDIRECT3DDEVICE8		m_pd3dDevice;				// d3d device
	WNDCLASSEX				m_wc;						// Needed for HWND
	HWND					hWnd;						// Hidden device window (for D3D)
	LPD3DXRENDERTOSURFACE	m_pRenderToSurface;			// Renderer
	LPDIRECT3DSURFACE8		m_pRenderTargetSurfaceLeft; // The surface of the render target
	LPDIRECT3DSURFACE8		m_pRenderTargetSurfaceRight;// The surface of the render target
	LPDIRECT3DVERTEXBUFFER8 m_pVB;						// Buffer to hold vertices
	LPDIRECT3DVERTEXBUFFER8 m_pVB_background;			// Buffer to hold vertices
	unsigned long			m_hLightningVertexShader;	// Vertex shader handle
	unsigned long			m_hLightningPixelShader;	// Pixel shader handle
	LPDIRECT3DTEXTURE8      m_pTexture;					// Our texture
	LPDIRECT3DTEXTURE8      m_pTexture_environment;		// Our texture
	LONG					m_lVidPitch;				// Video width

	// Log file
	ofstream *m_logfile;

	// Direct Show objects
    int m_iImageHeight;	                // The current image height
    int m_iImageWidth;	                // And current image width
    int m_iRepeatTime;                  // Time in msec between frames
    const int m_iDefaultRepeatTime;     // Initial m_iRepeatTime
    CCritSec m_cSharedState;	        // Lock on m_rtSampleTime and m_Ball
    BOOL m_bZeroMemory;                 // Do we need to clear the buffer
    CRefTime m_rtSampleTime;	        // The time stamp for each sample

}; // CBallStream
	
