//------------------------------------------------------------------------------
// File: ContProp.cpp
//
// Desc: DirectShow sample code - implementation of property page class
//       for the contrast filter.
//
// Copyright (c) 1992-2001 Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------------------------


#include <streams.h>
#include <commctrl.h>
#include <olectl.h>
#include <memory.h>
#include "resource.h"
#include "VirtualSource.h"
#include "VirtualSourceProp.h"


//
// CreateInstance
//
// This goes in the factory template table to create new filter instances
//
CUnknown * WINAPI CVirtualSourceProperties::CreateInstance(LPUNKNOWN lpunk, HRESULT *phr)
{
    CUnknown *punk = new CVirtualSourceProperties(lpunk, phr);
    if (punk == NULL) {
	*phr = E_OUTOFMEMORY;
    }
    return punk;

} // CreateInstance


//
// Constructor
//
CVirtualSourceProperties::CVirtualSourceProperties(LPUNKNOWN pUnk, HRESULT *phr) :
    CBasePropertyPage(NAME("VirtualSource Property Page"),pUnk,
                      IDD_VIRTUALSOURCEPROP,
                      IDS_TITLE)
    //m_pContrast(NULL)
{
    InitCommonControls();

} // (Constructor)


//
// SetDirty
//
// Sets m_bDirty and notifies the property page site of the change
//
void CVirtualSourceProperties::SetDirty()
{
    m_bDirty = TRUE;
    if (m_pPageSite) {
        m_pPageSite->OnStatusChange(PROPPAGESTATUS_DIRTY);
    }

} // SetDirty


//
// OnReceiveMessage
//
// Virtual method called by base class with Window messages
//
BOOL CVirtualSourceProperties::OnReceiveMessage(HWND hwnd,
                                           UINT uMsg,
                                           WPARAM wParam,
                                           LPARAM lParam)
{
    switch (uMsg)
    {
        case WM_INITDIALOG:
        {
	    m_hwndSlider = CreateSlider(hwnd, 50);
		m_hwndSlider2 = CreateSlider(hwnd, 152);
		m_hwndSlider3 = CreateSlider(hwnd, 254);
	    ASSERT(m_hwndSlider);
		ASSERT(m_hwndSlider2);
		ASSERT(m_hwndSlider3);
            return (LRESULT) 1;
        }
        case WM_VSCROLL:
        {
	    ASSERT(m_hwndSlider);
		ASSERT(m_hwndSlider2);
		ASSERT(m_hwndSlider3);
	    OnSliderNotification(wParam);
            return (LRESULT) 1;
        }

        case WM_COMMAND:
        {
	    if (LOWORD(wParam) == IDB_DEFAULT)
            {
	        SendMessage(m_hwndSlider, TBM_SETPOS, TRUE, 0L);
			SendMessage(m_hwndSlider2, TBM_SETPOS, TRUE, 0L);
			SendMessage(m_hwndSlider3, TBM_SETPOS, TRUE, 0L);
                SetDirty();
	    }
            return (LRESULT) 1;
        }

        case WM_DESTROY:
        {
	    DestroyWindow(m_hwndSlider);
		DestroyWindow(m_hwndSlider2);
		DestroyWindow(m_hwndSlider3);
            return (LRESULT) 1;
        }

    }
    return CBasePropertyPage::OnReceiveMessage(hwnd,uMsg,wParam,lParam);

} // OnReceiveMessage


//
// OnConnect
//
// Called when the property page connects to a filter
//
HRESULT CVirtualSourceProperties::OnConnect(IUnknown *pUnknown)
{
    //ASSERT(m_pContrast == NULL);

 /*   HRESULT hr = pUnknown->QueryInterface(IDD_VIRTUALSOURCEPROP, (void **) &m_pContrast);
    if (FAILED(hr)) {
        return E_NOINTERFACE;
    }
*/
    return NOERROR;

} // OnConnect


//
// OnDisconnect
//
// Called when we're disconnected from a filter
//
HRESULT CVirtualSourceProperties::OnDisconnect()
{
    // Release of Interface after setting the appropriate contrast value

/*    if (m_pContrast == NULL) {
        return E_UNEXPECTED;
    }
*/
   // m_pContrast->Release();
   // m_pContrast = NULL;
    return NOERROR;

} // OnDisconnect


//
// OnDeactivate
//
// We are being deactivated
//
HRESULT CVirtualSourceProperties::OnDeactivate(void)
{
    // Remember the present contrast level for the next activate

    return NOERROR;

} // OnDeactivate


//
// OnApplyChanges
//
// Changes made should be kept. Change the m_cContrastOnExit variable
//
HRESULT CVirtualSourceProperties::OnApplyChanges()
{
    m_bDirty = FALSE;
    return(NOERROR);

} // OnApplyChanges


//
// CreateSlider
//
// Create the slider (common control) to allow the user to adjust contrast
//
HWND CVirtualSourceProperties::CreateSlider(HWND hwndParent, int x)
{
    ULONG Styles = WS_CHILD | WS_VISIBLE | WS_TABSTOP | TBS_VERT | TBS_BOTH | WS_GROUP;
    HWND hwndSlider = CreateWindow( TRACKBAR_CLASS,
				    TEXT(""),
				    Styles,
				    x, 35,
				    39, 100,
				    hwndParent,
				    NULL,
				    g_hInst,
				    NULL);
    if (hwndSlider == NULL) {
	    // DWORD dwErr = GetLastError();
	    return NULL;
    }

    // Set the initial range for the slider
 /*   SendMessage(hwndSlider, TBM_SETRANGE, TRUE, MAKELONG(MinContrastLevel, MaxContrastLevel) );

    // Set a tick at zero
    SendMessage(hwndSlider, TBM_SETTIC, 0, 0L);

    // Set the initial slider position
    SendMessage(hwndSlider, TBM_SETPOS, TRUE, m_clLevel);
*/
    return hwndSlider;

} // CreateSlider


//
// OnSliderNotification
//
// Handle the notification messages from the slider control
//
void CVirtualSourceProperties::OnSliderNotification(WPARAM wParam)
{
    switch (wParam) {
        case TB_BOTTOM:
            SetDirty();
//	    SendMessage(m_hwndSlider, TBM_SETPOS, TRUE, (LPARAM) MinContrastLevel);
	    break;

        case TB_TOP:
            SetDirty();
 //   	    SendMessage(m_hwndSlider, TBM_SETPOS, TRUE, (LPARAM) MaxContrastLevel);
            break;

        case TB_PAGEDOWN:
        case TB_PAGEUP:
            break;

        case TB_THUMBTRACK:
        case TB_THUMBPOSITION:
        case TB_ENDTRACK:
        {
            SetDirty();
			// get l-value from slider
//            signed char Level = (signed char) SendMessage(m_hwndSlider, TBM_GETPOS, 0, 0L);
//	        pIContrast()->put_lLevel(Level, 0);
			// get alha-value from slider
//			Level = (signed char) SendMessage(m_hwndSlider2, TBM_GETPOS, 0, 0L);
//	        pIContrast()->put_alphaLevel(Level, 0);
			// get alha-value from slider
//			Level = (signed char) SendMessage(m_hwndSlider3, TBM_GETPOS, 0, 0L);
//	        pIContrast()->put_betaLevel(Level, 0);
        }
	break;

        // Default handling of these messages is ok
        //case TB_THUMBTRACK:
        case TB_LINEDOWN:
        case TB_LINEUP:
	    break;
    }

} // OnSliderNotification
